#include "m6955.h"

/* Address of slave devices */
#define M6955_ADDRESS              0x20
#define AM  0b10000000
#define FM  0b11000000

typedef uintptr_t APP_I2C_DRIVER_HANDLE;
typedef uintptr_t APP_I2C_BUFFER_HANDLE;

APP_I2C_DRIVER_HANDLE   drvI2CHandle_Master;
APP_I2C_DRIVER_HANDLE   drvI2CTxRxBufferHandle;

DRV_I2C_BUFFER_EVENT i2cOpStatus;

/* I2C Driver TX buffer  */
uint8_t         TXbuffer[10];

void I2CMasterOpStatusCb ( DRV_I2C_BUFFER_EVENT event,
                           DRV_I2C_BUFFER_HANDLE bufferHandle,
                           uintptr_t context);

void I2CSlaveOpStatusCb  ( DRV_I2C_BUFFER_EVENT event,
                           DRV_I2C_BUFFER_HANDLE bufferHandle,
                           uintptr_t context);

DRV_I2C_BUFFER_EVENT Check_Transfer_Status(DRV_HANDLE drvOpenHandle, DRV_I2C_BUFFER_HANDLE drvBufferHandle);

bool Send_First_Data(uint8_t reg, uint8_t value){

    if ( (drvI2CTxRxBufferHandle == (DRV_I2C_BUFFER_HANDLE) NULL) || 
        (Check_Transfer_Status(drvI2CHandle_Master, drvI2CTxRxBufferHandle) == DRV_I2C_BUFFER_EVENT_COMPLETE) || 
            (Check_Transfer_Status(drvI2CHandle_Master, drvI2CTxRxBufferHandle) == DRV_I2C_BUFFER_EVENT_ERROR) )
    { 
        TXbuffer[0]= reg; TXbuffer[1]= value;
        drvI2CTxRxBufferHandle = DRV_I2C_Transmit ( drvI2CHandle_Master, M6955_ADDRESS, &TXbuffer, 2, NULL);
        return true;
    }   
    else  return false;
}

bool Send_Data(uint8_t reg, uint8_t value){

    if ( Check_Transfer_Status(drvI2CHandle_Master, drvI2CTxRxBufferHandle) == DRV_I2C_BUFFER_EVENT_COMPLETE)
    { 
        TXbuffer[0]= reg; TXbuffer[1]= value;
        drvI2CTxRxBufferHandle = DRV_I2C_Transmit ( drvI2CHandle_Master, M6955_ADDRESS, &TXbuffer, 2, NULL);
        return true;
    }   
    else  return false;
}

bool M6955_Init(void){
    
    switch(InitState)
    {
         case INIT_OPEN:
        {           
            DelayMs(500);
            drvI2CHandle_Master = DRV_I2C_Open(DRV_I2C_INDEX_0, DRV_IO_INTENT_READWRITE);
     
            /* event-handler set up receive callback from DRV_I2C_Tasks */
            DRV_I2C_BufferEventHandlerSet(drvI2CHandle_Master, I2CMasterOpStatusCb, i2cOpStatus );
                      
            /* Write Transaction to M6955 */
            if(drvI2CHandle_Master != (DRV_HANDLE) NULL)
            {
                InitState = INIT_REG00;
            }
            else
            {
                InitState = INIT_ERROR;
            }
            break;
        }       
            
        case INIT_REG00:
        {
            if(Send_First_Data(0x00, 0b10000000))//b7=Power ON, b6=AM mode
                InitState = INIT_REG06;
            break;
        }
        
        case INIT_REG06:
        {
            if(Send_Data(0x06, 0b11000001)) // b7:2=volume 48/64, b1=radio mode, b0=single speaker
                InitState = INIT_REG07;
            break;
        }
            
        case INIT_REG07:
        {
            if(Send_Data(0x07, 0b00110001)) // b5=base boost, b3:2=auto stereo
                InitState = INIT_REG09;
            break;
        }
        
        case INIT_REG09:
        {
            if(Send_Data(0x09, 0b00000111)) // b3=vol control resistor, b2=Xtal
                 InitState = INIT_COMPLETED;           
            break;
        }
        
        case INIT_COMPLETED:
        {
            if ( Check_Transfer_Status(drvI2CHandle_Master, drvI2CTxRxBufferHandle) == DRV_I2C_BUFFER_EVENT_COMPLETE)
            { 
                return true;
            }
            break;
        }
                
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
    return false;
}

bool M6955_Freq(uint8_t band, uint16_t freq){
    
    uint16_t ch;
    
    if(band==AM)   ch = (freq / 9) * 3;
    else if(band==FM)    ch=((freq-3000)*10)/25;
    
    switch(FreqState)
    {
        case FREQ_REG03:
        {
            DelayMs(300);
            if(Send_Data(0x03, (ch & 0xff)))    //b4:0=ch higher bits
                FreqState = FREQ_REG02;
            break;
        }
        
        case FREQ_REG02:
        {
            if(Send_Data(0x02, ((ch >> 8) | 0x40))) // b7:0=ch lower bits
                FreqState = FREQ_REG01_1;
            break;
        }
            
        case FREQ_REG01_1:
        {
            if(Send_Data(0x00,  band)) // b6=mode
                FreqState = FREQ_REG01_2;
            break;
        }
        
        case FREQ_REG01_2:
        {
            DelayMs(1);
            if(Send_Data(0x00, (band | 0b00100000))) // b6=mode, b5=Trigger tune process
                 FreqState = FREQ_REG01_3;           
            break;
        }

        case FREQ_REG01_3:
        {
            DelayMs(1);
            if(Send_Data(0x00, band)) // b6= mode
                 FreqState = FREQ_COMPLETED;           
            break;
        }
                
        case FREQ_COMPLETED:
        {
            if ( Check_Transfer_Status(drvI2CHandle_Master, drvI2CTxRxBufferHandle) == DRV_I2C_BUFFER_EVENT_COMPLETE)
            { 
                return true;
            }
            break;
        }
                
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
    return false;
}

//****************************************************************************/
//  Function: Check_Transfer_Status
//
//  Returns the status of Buffer operation from the driver; The application
//  can probe this function to see if the status of a particular I2C
//  transfer is in its execution
//****************************************************************************/
DRV_I2C_BUFFER_EVENT Check_Transfer_Status(DRV_HANDLE drvOpenHandle, DRV_I2C_BUFFER_HANDLE drvBufferHandle)
{
    return (DRV_I2C_TransferStatusGet  (drvI2CHandle_Master, drvBufferHandle));
}

//****************************************************************************/
//  Function: Master Callback Function 
//
//  Callback from DRV_I2C_Tasks when I2C is configured in Master mode     
//****************************************************************************/

void I2CMasterOpStatusCb ( DRV_I2C_BUFFER_EVENT event,
                           DRV_I2C_BUFFER_HANDLE bufferHandle,
                           uintptr_t context)
{
    /*
    static uint32_t successCount = 0;
    
    switch (event)
    {
        case DRV_I2C_BUFFER_EVENT_COMPLETE:
            successCount++;
            Nop();
            break;
        case DRV_I2C_BUFFER_EVENT_ERROR:
            successCount--;
            break;
        default:
            break;
    }
     */
}

//****************************************************************************/
//  Function: Slave Callback Function 
//
//  Callback from DRV_I2C_Tasks when I2C is configured in Slave mode     
//****************************************************************************/

void I2CSlaveOpStatusCb ( DRV_I2C_BUFFER_EVENT event,
                          DRV_I2C_BUFFER_HANDLE bufferHandle,
                          uintptr_t context)
{
    Nop();
}

/* *****************************************************************************
 End of File
 */
